@@ -1,16 +1,18 @@
 use std::io;
 use std::io::Write;
 use std::net::{IpAddr, SocketAddr, TcpListener};
+use std::path::{Path, PathBuf};
 use std::thread;
 use std::time::Duration;
 
+use actix_files::NamedFile;
 use actix_web::web;
 use actix_web::{http::header::ContentType, Responder};
 use actix_web::{middleware, App, HttpRequest, HttpResponse};
-use anyhow::{bail, Result};
-use clap::{crate_version, Clap, IntoApp};
-use clap_generate::generators::{Bash, Elvish, Fish, PowerShell, Zsh};
-use clap_generate::{generate, Shell};
+use actix_web_httpauth::middleware::HttpAuthentication;
+use anyhow::Result;
+use clap::{crate_version, IntoApp, Parser};
+use clap_complete::generate;
 use log::{error, warn};
 use qrcodegen::{QrCode, QrCodeEcc};
 use yansi::{Color, Paint};
@@ -32,19 +34,9 @@ fn main() -> Result<()> {
     let args = args::CliArgs::parse();
 
     if let Some(shell) = args.print_completions {
-        let mut clap_app = args::CliArgs::into_app();
-        match shell {
-            Shell::Bash => generate::<Bash, _>(&mut clap_app, "miniserve", &mut std::io::stdout()),
-            Shell::Elvish => {
-                generate::<Elvish, _>(&mut clap_app, "miniserve", &mut std::io::stdout())
-            }
-            Shell::Fish => generate::<Fish, _>(&mut clap_app, "miniserve", &mut std::io::stdout()),
-            Shell::PowerShell => {
-                generate::<PowerShell, _>(&mut clap_app, "miniserve", &mut std::io::stdout())
-            }
-            Shell::Zsh => generate::<Zsh, _>(&mut clap_app, "miniserve", &mut std::io::stdout()),
-            _ => bail!("Invalid shell provided!"),
-        }
+        let mut clap_app = args::CliArgs::command();
+        let app_name = clap_app.get_name().to_string();
+        generate(shell, &mut clap_app, app_name, &mut io::stdout());
         return Ok(());
     }
 
@@ -105,15 +97,8 @@ async fn run(miniserve_config: MiniserveConfig) -> Result<(), ContextualError> {
         ContextualError::IoError("Failed to resolve path to be served".to_string(), e)
     })?;
 
-    if let Some(index_path) = &miniserve_config.index {
-        let has_index: std::path::PathBuf = [&canon_path, index_path].iter().collect();
-        if !has_index.exists() {
-            error!(
-                "The file '{}' provided for option --index could not be found.",
-                index_path.to_string_lossy()
-            );
-        }
-    }
+    check_file_exists(&canon_path, &miniserve_config.index);
+
     let path_string = canon_path.to_string_lossy();
 
     println!(
@@ -180,10 +165,7 @@ async fn run(miniserve_config: MiniserveConfig) -> Result<(), ContextualError> {
                 Some(_) => format!("https://{}", addr),
                 None => format!("http://{}", addr),
             })
-            .map(|url| match miniserve_config.random_route {
-                Some(ref random_route) => format!("{}/{}", url, random_route),
-                None => url,
-            })
+            .map(|url| format!("{}{}", url, miniserve_config.route_prefix))
             .collect::<Vec<_>>()
     };
 
@@ -204,18 +186,16 @@ async fn run(miniserve_config: MiniserveConfig) -> Result<(), ContextualError> {
             .app_data(inside_config.clone())
             .wrap_fn(errors::error_page_middleware)
             .wrap(middleware::Logger::default())
-            .route(
-                &format!("/{}", inside_config.favicon_route),
-                web::get().to(favicon),
-            )
-            .route(&format!("/{}", inside_config.css_route), web::get().to(css))
+            .route(&inside_config.favicon_route, web::get().to(favicon))
+            .route(&inside_config.css_route, web::get().to(css))
             .service(
-                web::scope(inside_config.random_route.as_deref().unwrap_or(""))
-                    // we should use `actix_web_httpauth::middleware::HttpAuthentication`
-                    // but it is unfortuantrly broken
-                    // see: https://github.com/actix/actix-extras/issues/127
-                    // TODO replace this when fixed upstream
-                    .wrap_fn(auth::auth_middleware)
+                web::scope(&inside_config.route_prefix)
+                    .wrap(middleware::Condition::new(
+                        !inside_config.auth.is_empty(),
+                        actix_web::middleware::Compat::new(HttpAuthentication::basic(
+                            auth::handle_auth,
+                        )),
+                    ))
                     .configure(|c| configure_app(c, &inside_config)),
             )
             .default_service(web::get().to(error_404))
@@ -245,7 +225,7 @@ async fn run(miniserve_config: MiniserveConfig) -> Result<(), ContextualError> {
     println!("Serving path {}", Color::Yellow.paint(path_string).bold());
 
     println!(
-        "Availabe at (non-exhaustive list):\n    {}\n",
+        "Available at (non-exhaustive list):\n    {}\n",
         display_urls
             .iter()
             .map(|url| Color::Green.paint(url).bold().to_string())
@@ -279,6 +259,19 @@ async fn run(miniserve_config: MiniserveConfig) -> Result<(), ContextualError> {
         .map_err(|e| ContextualError::IoError("".to_owned(), e))
 }
 
+fn check_file_exists(canon_path: &Path, file_option: &Option<PathBuf>) {
+    if let Some(file_path) = file_option {
+        let file_path: &Path = file_path.as_ref();
+        let has_file: std::path::PathBuf = [canon_path, file_path].iter().collect();
+        if !has_file.exists() {
+            error!(
+                "The file '{}' provided for option --index could not be found.",
+                file_path.to_string_lossy(),
+            );
+        }
+    }
+}
+
 /// Allows us to set low-level socket options
 ///
 /// This mainly used to set `set_only_v6` socket option
@@ -299,7 +292,7 @@ fn create_tcp_listener(addr: SocketAddr) -> io::Result<TcpListener> {
 fn configure_header(conf: &MiniserveConfig) -> middleware::DefaultHeaders {
     conf.header.iter().flatten().fold(
         middleware::DefaultHeaders::new(),
-        |headers, (header_name, header_value)| headers.header(header_name, header_value),
+        |headers, (header_name, header_value)| headers.add((header_name, header_value)),
     )
 }
 
@@ -308,8 +301,25 @@ fn configure_app(app: &mut web::ServiceConfig, conf: &MiniserveConfig) {
     let files_service = || {
         let files = actix_files::Files::new("", &conf.path);
         let files = match &conf.index {
-            Some(index_file) => files.index_file(index_file.to_string_lossy()),
-            None => files,
+            Some(index_file) => {
+                if conf.spa {
+                    files
+                        .index_file(index_file.to_string_lossy())
+                        .default_handler(
+                            NamedFile::open(&conf.path.join(index_file))
+                                .expect("Cant open SPA index file."),
+                        )
+                } else {
+                    files.index_file(index_file.to_string_lossy())
+                }
+            }
+            None => {
+                if conf.spa {
+                    unreachable!("This can't be reached since we require --index to be provided if --spa is given via clap");
+                } else {
+                    files
+                }
+            }
         };
         let files = match conf.show_hidden {
             true => files.use_hidden_files(),
@@ -320,7 +330,6 @@ fn configure_app(app: &mut web::ServiceConfig, conf: &MiniserveConfig) {
             .files_listing_renderer(listing::directory_listing)
             .prefer_utf8(true)
             .redirect_to_slash_directory()
-            .default_handler(web::to(error_404))
     };
 
     if !conf.path.is_file() {
@@ -344,14 +353,14 @@ async fn favicon() -> impl Responder {
     let logo = include_str!("../data/logo.svg");
     HttpResponse::Ok()
         .insert_header(ContentType(mime::IMAGE_SVG))
-        .message_body(logo.into())
+        .body(logo)
 }
 
 async fn css() -> impl Responder {
     let css = include_str!(concat!(env!("OUT_DIR"), "/style.css"));
     HttpResponse::Ok()
         .insert_header(ContentType(mime::TEXT_CSS))
-        .message_body(css.into())
+        .body(css)
 }
 
 // Prints to the console two inverted QrCodes side by side.
